/* ScummVM - Scumm Interpreter
 * Copyright (C) 2003 The ScummVM project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /cvsroot/scummvm/scummvm/kyra/kyra.h,v 1.1 2004/04/09 12:36:06 ender Exp $
 *
 */

#ifndef KYRA_H
#define KYRA_H

#include "common/stdafx.h"
#include "common/scummsys.h"
#include "base/engine.h"
#include "base/gameDetector.h"
#include "common/util.h"

#include "sound/mididrv.h"

namespace Kyra {

class Resourcemanager;
class Palette;
class CPSImage;
class Font;
class VMContext;

class KyraEngine : public Engine {

	void errorString( const char *buf_input, char *buf_output);

protected:
	void go();
	void shutdown();

public:

	KyraEngine(GameDetector *detector, OSystem *syst);
	virtual ~KyraEngine();
	
	Resourcemanager* resManager(void) { return _resMgr; }
	MidiDriver* midiDriver(void) { return _midiDriver; }
	
protected:

	MidiDriver* _midiDriver; // the MidiDriver
	Resourcemanager* _resMgr; // our 'great' resource manager
	VMContext* _currentScript; // our current script
	VMContext* _npcScript; // script from NPCs
	
public:
		
	void setCurrentPalette(Palette* pal, bool delNextTime = true);
	void setTempPalette(Palette* pal);
	void upadateCurrentPal(void);
	
protected:
	bool _delPalNextTime;
	Palette* _currentPal;
	Font* _font;
	
public:
	void drawCursor(bool draw) { _drawCursor = draw; }
	void drawCursor(void);
	
protected:
	bool _drawCursor;
	CPSImage* _mouse;
 	CPSImage* _items; // the mouse cursors
	
public:
	void updateScreen(void);
	uint8* plane(void) { return _plane; }

protected:
	uint8* _plane;
	
public:
	enum EngineState {
		kStateIdle = 0,
		kStateWalking = 1,
		kStateTalking = 2
	};
	
	enum Cursor {
		kArrowCursor = 0,
		kUpCursor = 1,
		kRightCursor = 2,
		kDownCursor = 3,
		kLeftCursor = 4,
		kNoCursor = 5,
		kTurnLeftCursor = 6,
		kTurnRightCursor = 7,
		kCloseCursor = 8,
		kItemCursor = 9
	};
	
	enum ScriptIndex {
		kSetupScene = 0,
		kClickEvent = 1,
		kActorEvent = 2,
		kEnterEvent = 4,
		kExitEvent = 5,
		kLoadResources = 7
	};
	
protected:

};

} // End of namespace Kyra

#endif
