#ifndef MOVIES_H
#define MOVIES_H

#include "resource.h"

namespace Kyra {

	// a generic movie
	class Movie {
	
	public:
	
		virtual ~Movie() {}
		
		virtual void render(uint8* plane, uint16 planepitch, uint16 planeheight) = 0;
		
		virtual const uint8* loadFrame(uint16 frame, uint16* width = 0, uint16* height = 0) = 0;
		virtual uint16 countFrames(void) { return _frameCount; }
		
		virtual bool hasPalette(void) { return (_ownPalette != 0); }
		virtual Palette* palette(void) { return _ownPalette; }
		
		virtual void setTransparency(int8 color) { _transparency = color; }
		virtual void setDelay(uint32 delay) { _delay = delay; }
		
		virtual void setRenderedFrames(uint16 start_frame, uint16 end_frame)
			{ _start_frame = start_frame; _end_frame = end_frame; }
	
	protected:
		uint16 _frameCount;
		Palette* _ownPalette;
		int8 _transparency;
		uint32 _delay;
		uint16 _start_frame, _end_frame;
	};
	
	// movie format for Kyrandia 1
	// there is also a new WSA Format for Kyrandia 2
	// which i will implement in future
	class WSAMovieV1 : public Movie {
	
	public:
	
		WSAMovieV1(uint8* data, uint32 size);
		~WSAMovieV1();
		
		const uint8* loadFrame(uint16 frame, uint16* width, uint16* height);
		void render(uint8* plane, uint16 planepitch, uint16 planeheight);
		
		void setRenderedFrames(uint16 start_frame, uint16 end_frame)
			{ _currentFrameNum = _start_frame = start_frame; _end_frame = end_frame; }
	protected:
	
		uint8* _buffer;
	
#pragma START_PACK_STRUCTS	
		struct WSAHeader {
			uint16 _numFrames; // All right
			uint16 _width; // should be right
			uint16 _height; // should be right
			uint8 _xPos; // could be wrong
			uint8 _yPos; // could be wrong
			uint16 _delta; // could be wrong
			uint16 _type; // should be right
		} GCC_PACK _wsaHeader;
#pragma END_PACK_STRUCTS

		uint32* _offsetTable;
		
		uint8* _currentFrame;
		uint16 _prefetchedFrame;
		
		uint16 _currentFrameNum;
		uint32 _lastFrameChange;
	};
} // end of namespace Kyra

#endif

