#include "resource.h"

#include "common/file.h"

namespace Kyra {
	PAKFile::PAKFile(const Common::String& file) {
		File pakfile;

		if (!pakfile.open(file.c_str())) {
			warning("couldn't open file '%s'", file.c_str());
			return;
		}

		uint32 filesize = pakfile.size();
		_buffer = new uint8[filesize];
		assert(_buffer);

		pakfile.read(_buffer, filesize);

		pakfile.close();

		// works with the file
		uint32 pos = 0, startoffset = 0, endoffset = 0;

		startoffset = *(reinterpret_cast<uint32*>((_buffer + pos)));
		pos += 4;

		for (;pos < filesize;)
		{
			PakChunk* chunk = new PakChunk;
			assert(chunk);

			// saves the name
			chunk->_name = reinterpret_cast<const char*>(_buffer + pos);
			pos += strlen(chunk->_name) + 1;

			if(!chunk->_name)
				break;

			endoffset = *(reinterpret_cast<uint32*>((_buffer + pos)));
			pos += 4;

			chunk->_data = _buffer + startoffset;
			chunk->_size = endoffset - startoffset;

			startoffset = endoffset;

			_files.push_back(chunk);
		}
	}

	PAKFile::~PAKFile()	{
		delete [] _buffer;
		_buffer = 0;

		Common::List<PakChunk*>::iterator start = _files.begin();

		for (;start != _files.end();++start)
		{
			delete *start;
			*start = 0;
		}
	}

	const uint8* PAKFile::getFile(const char* file)	{
		for (Common::List<PakChunk*>::iterator start = _files.begin(); start != _files.end(); ++start) {
			if (!scumm_stricmp((*start)->_name, file)) {
				return (*start)->_data;
			}
		}

		return 0;
	}
	
	uint32 PAKFile::getFileSize(const char* file) {
		for (Common::List<PakChunk*>::iterator start = _files.begin(); start != _files.end(); ++start) {
			if (!scumm_stricmp((*start)->_name, file)) {
				return (*start)->_size;
			}
		}
		
		return 0;
	}
} // end of namespace Kyra
