#include "resource.h"
#include "movies.h"
#include "VMContext.h"
#include "music.h"

#include "common/file.h"

namespace Kyra {
	Resourcemanager::Resourcemanager(KyraEngine* engine, const char* gamedir) {
		_gameDir = gamedir;
		_engine = engine;
		
		// prefetches all PAK Files
		
		// ugly a hardcoded list
		// TODO: use the FS Backend to get all .PAK Files and load them
		static const char* kyraFilelist[] = {
  			"A_E.PAK", "DAT.PAK", "F_L.PAK", "MAP_5.PAK", "MSC.PAK", "M_S.PAK",
     		"S_Z.PAK", "WSA1.PAK", "WSA2.PAK", "WSA3.PAK", "WSA4.PAK", "WSA5.PAK",
			"WSA6.PAK", 0
		};
		
		for (uint32 tmp = 0; kyraFilelist[tmp]; ++tmp)	{
				
			// prefetch file
			PAKFile* file = new PAKFile(getPath() + kyraFilelist[tmp]);
			assert(file);			
     
			if (file->isVaild())		
				_pakfiles.push_back(file);
			else
				warning("couldn't load file '%s' correctly", kyraFilelist[tmp]);
		}
	}
	
	Resourcemanager::~Resourcemanager() {
		Common::List<PAKFile*>::iterator start = _pakfiles.begin();

		for (;start != _pakfiles.end(); ++start) {
			delete *start;
			*start = 0;
		}
	}
	
	uint8* Resourcemanager::fileData(const char* file, uint32* size) {
		uint8* buffer = 0;
		File file_;
		
		// test to open it in the main dir
		if (file_.open((getPath() + file).c_str())) {
		
			*size = file_.size();
			
			buffer = new uint8[*size];
			assert(buffer);
			
			file_.read(buffer, *size);
			
			file_.close();
		
		} else {
			// opens the file in a PAK File
			Common::List<PAKFile*>::iterator start = _pakfiles.begin();
			
			for (;start != _pakfiles.end(); ++start) {
			
				*size = (*start)->getFileSize(file);
				
				if (!*size)
					continue;
					
				buffer = new uint8[*size];
				assert(buffer);
				
				// creates a copy of the file
				memcpy(buffer, (*start)->getFile(file), *size);
				
				break;
			}
			
		}
		
		if (!buffer || !(*size)) {
			warning("couldn't find file '%s'", file);
		}
		
		return buffer;
	}
	
	Palette* Resourcemanager::loadPalette(const char* file)	{
		uint32 size = 0;
		uint8* buffer = 0;
		buffer = fileData(file, &size);
		if (!buffer)
			return 0;
		return new Palette(buffer, size);
	}
	
	CPSImage* Resourcemanager::loadImage(const char* file) {
		uint32 size = 0;
		uint8* buffer = 0;
		buffer = fileData(file, &size);
		if (!buffer)
			return 0;
		return new CPSImage(buffer, size);
	}
	
	Font* Resourcemanager::loadFont(const char* file) {
		uint32 size = 0;
		uint8* buffer = 0;
		buffer = fileData(file, &size);
		if (!buffer)
			return 0;
		return new Font(buffer, size);
	}
	
	Movie* Resourcemanager::loadMovie(const char* file) {
		// TODO: we have to check the Extenion to create the right movie
		uint32 size = 0;
		uint8* buffer = 0;
		buffer = fileData(file, &size);
		if (!buffer)
			return 0;
		return new WSAMovieV1(buffer, size);
	}
	
	MusicFile* Resourcemanager::loadMusic(const char* file) {
		int32 strlgt = strlen(file);
		
		// only XMI is supported
		if (file[strlgt-3] != 'X' || file[strlgt-2] != 'M' || file[strlgt-1] != 'I')
			return 0;

		uint32 size = 0;
		return new XMIDI(fileData(file, &size), size, _engine);
	}
	
	VMContext* Resourcemanager::loadScript(const char* file) {
		VMContext* context = new VMContext(_engine);
		context->loadScript(file);
		return context;
	}
	
	Common::String Resourcemanager::getPath(void) {
		assert(_gameDir);
		int32 len = strlen(_gameDir);
		
		if(len < 1)
			error("no vaild gamedir");
			
		// tests for an path seperator at the end
		if (_gameDir[len - 1] == '\\') {
			return string(_gameDir);
		} else if (_gameDir[len - 1 ] == '/') {
			return string(_gameDir);
		}
	
		// creates a path seperator at the end
		// we are always using the path seperator from the system
		// even if Windows shoudl accept '/'
#ifdef WIN32	
		return string(_gameDir) + "\\";
#else
		return string(_gameDir) + "/";
#endif
	}
} // end of namespace Kyra

